@@ -7,7 +7,7 @@ from io import StringIO
 import pytest
 
 import isort
-from isort import Config, exceptions
+from isort import Config, api, exceptions
 
 
 def test_semicolon_ignored_for_dynamic_lines_after_import_issue_1178():
@@ -213,6 +213,19 @@ import os
 """
     )
 
+    # issue 1838: don't append in middle of class
+    assert isort.check_code(
+        '''class C:
+    """a
+
+    """
+    # comment
+''',
+        append_only=True,
+        add_imports=["from __future__ import annotations"],
+        show_diff=True,
+    )
+
 
 def test_isort_supports_shared_profiles_issue_970():
     """Test to ensure isort provides a way to use shared profiles.
@@ -483,6 +496,43 @@ d = 1
 # isort: dict
 y = {"b": "c", "z": "z"}"""
     )
+    assert api.sort_stream(
+        input_stream=StringIO(
+            """
+import a
+import x
+
+# isort: list
+__all__ = ["b", "a", "b"]
+
+# isort: unique-list
+__all__ = ["b", "a", "b"]
+
+# isort: tuple
+__all__ = ("b", "a", "b")
+
+# isort: unique-tuple
+__all__ = ("b", "a", "b")
+
+# isort: set
+__all__ = {"b", "a", "b"}
+
+
+def method():
+    # isort: list
+    x = ["b", "a"]
+
+
+# isort: assignments
+d = 1
+b = 2
+a = 3
+
+# isort: dict
+y = {"z": "z", "b": "b", "b": "c"}""",
+        ),
+        output_stream=StringIO(),
+    )
 
 
 def test_isort_allows_setting_import_types_issue_1181():
@@ -856,7 +906,6 @@ def test_api_to_allow_custom_diff_and_output_stream_1583(capsys, tmpdir):
     to a different stream.
     See: https://github.com/PyCQA/isort/issues/1583
     """
-
     tmp_file = tmpdir.join("file.py")
     tmp_file.write("import b\nimport a\n")
 
@@ -877,6 +926,21 @@ def test_api_to_allow_custom_diff_and_output_stream_1583(capsys, tmpdir):
     isort_output.seek(0)
     assert isort_output.read().splitlines() == ["import a", "import b"]
 
+    # should still work with no diff produced
+    tmp_file2 = tmpdir.join("file2.py")
+    tmp_file2.write("import a\nimport b\n")
+
+    isort_diff2 = StringIO()
+    isort_output2 = StringIO()
+
+    isort.file(tmp_file2, show_diff=isort_diff2, output=isort_output2)
+
+    _, error = capsys.readouterr()
+    assert not error
+
+    isort_diff2.seek(0)
+    assert not isort_diff2.read()
+
 
 def test_autofix_mixed_indent_imports_1575():
     """isort should automatically fix import statements that are sent in
@@ -971,3 +1035,202 @@ def function():
     import numpy as np
 """
     )
+
+
+def test_isort_auto_detects_and_ignores_invalid_from_imports_issue_1688():
+    """isort should automatically detect and ignore incorrectly written from import statements
+    see: https://github.com/PyCQA/isort/issues/1688
+    """
+    assert (
+        isort.code(
+            """
+from package1 import alright
+from package2 imprt and_its_gone
+from package3 import also_ok
+"""
+        )
+        == """
+from package1 import alright
+
+from package2 imprt and_its_gone
+from package3 import also_ok
+"""
+    )
+
+
+def test_isort_allows_reversing_sort_order_issue_1645():
+    """isort allows reversing the sort order for those who prefer Z or longer imports first.
+    see: https://github.com/PyCQA/isort/issues/1688
+    """
+    assert (
+        isort.code(
+            """
+from xxx import (
+    g,
+    hi,
+    def,
+    abcd,
+)
+""",
+            profile="black",
+            reverse_sort=True,
+            length_sort=True,
+            line_length=20,
+        )
+        == """
+from xxx import (
+    abcd,
+    def,
+    hi,
+    g,
+)
+"""
+    )
+
+
+def test_isort_can_push_star_imports_above_others_issue_1504():
+    """isort should provide a way to push star imports above other imports to avoid explicit
+    imports from being overwritten.
+    see: https://github.com/PyCQA/isort/issues/1504
+    """
+    assert (
+        (
+            isort.code(
+                """
+from ._bar import Any, All, Not
+from ._foo import a, *
+""",
+                star_first=True,
+            )
+        )
+        == """
+from ._foo import *
+from ._foo import a
+from ._bar import All, Any, Not
+"""
+    )
+
+
+def test_isort_can_combine_reverse_sort_with_force_sort_within_sections_issue_1726():
+    """isort should support reversing import order even with force sort within sections turned on.
+    See: https://github.com/PyCQA/isort/issues/1726
+    """
+    assert (
+        isort.code(
+            """
+import blaaa
+from bl4aaaaaaaaaaaaaaaa import r
+import blaaaaaaaaaaaa
+import bla
+import blaaaaaaa
+from bl1aaaaaaaaaaaaaa import this_is_1
+from bl2aaaaaaa import THIIIIIIIIIIIISS_is_2
+from bl3aaaaaa import less
+""",
+            length_sort=True,
+            reverse_sort=True,
+            force_sort_within_sections=True,
+        )
+        == """
+from bl2aaaaaaa import THIIIIIIIIIIIISS_is_2
+from bl1aaaaaaaaaaaaaa import this_is_1
+from bl4aaaaaaaaaaaaaaaa import r
+from bl3aaaaaa import less
+import blaaaaaaaaaaaa
+import blaaaaaaa
+import blaaa
+import bla
+"""
+    )
+
+
+def test_isort_can_turn_off_import_adds_with_action_comment_issue_1737():
+    assert (
+        isort.code(
+            """
+import os
+""",
+            add_imports=[
+                "from __future__ import absolute_imports",
+                "from __future__ import annotations",
+            ],
+        )
+        == """
+from __future__ import absolute_imports, annotations
+
+import os
+"""
+    )
+
+    assert isort.check_code(
+        """
+# isort: dont-add-imports
+import os
+""",
+        show_diff=True,
+        add_imports=[
+            "from __future__ import absolute_imports",
+            "from __future__ import annotations",
+        ],
+    )
+
+    assert (
+        isort.code(
+            """
+# isort: dont-add-import: from __future__ import annotations
+import os
+""",
+            add_imports=[
+                "from __future__ import absolute_imports",
+                "from __future__ import annotations",
+            ],
+        )
+        == """
+# isort: dont-add-import: from __future__ import annotations
+from __future__ import absolute_imports
+
+import os
+"""
+    )
+
+
+def test_sort_configurable_sort_issue_1732() -> None:
+    """Test support for pluggable isort sort functions."""
+    test_input = (
+        "from bob2.apples2 import aardvark as aardvark2\n"
+        "from bob.apples import aardvark \n"
+        "import module9\n"
+        "import module10\n"
+        "import module200\n"
+    )
+    assert isort.code(test_input, sort_order="native") == (
+        "import module10\n"
+        "import module200\n"
+        "import module9\n"
+        "from bob.apples import aardvark\n"
+        "from bob2.apples2 import aardvark as aardvark2\n"
+    )
+    assert (
+        isort.code(test_input, sort_order="natural")
+        == isort.code(test_input)
+        == (
+            "import module9\n"
+            "import module10\n"
+            "import module200\n"
+            "from bob2.apples2 import aardvark as aardvark2\n"
+            "from bob.apples import aardvark\n"
+        )
+    )
+    assert (
+        isort.code(test_input, sort_order="natural_plus")
+        == isort.code(test_input)
+        == (
+            "import module9\n"
+            "import module10\n"
+            "import module200\n"
+            "from bob2.apples2 import aardvark as aardvark2\n"
+            "from bob.apples import aardvark\n"
+        )
+    )
+    with pytest.raises(exceptions.SortingFunctionDoesNotExist):
+        isort.code(test_input, sort_order="round")
